#include "GetFinderSelection.h"
#include <AppleEvents.h>
#include <AEPackObject.h>

ProcessSerialNumber GetAddressOfFinder(void)
{
   ProcessSerialNumber   psn;
   ProcessInfoRec         theProc;

// Initialize the process serial number to specify no process.
   psn.highLongOfPSN = 0;
   psn.lowLongOfPSN = kNoProcess;
   
// Initialize the fields in the ProcessInfoRec, or we'll have memory 
// hits in random locations.
   theProc.processInfoLength = sizeof(ProcessInfoRec);
   theProc.processName = nil;
   theProc.processAppSpec = nil;
   theProc.processLocation = nil;
   
// Loop through all processes, looking for the Finder.
   while(true) 
   {
      GetNextProcess(&psn);
      GetProcessInformation(&psn, &theProc);
      if ((theProc.processType == 'FNDR') &&
          (theProc.processSignature == 'MACS'))
         break;
   }
   return(psn);
}



// tell application "Finder"
//      get selection
// end tell
//
AEDesc	GetFinderSelection(void)
{
	AppleEvent   	ae;
	AppleEvent   	reply;
	ProcessSerialNumber	finderSerial;
	OSType			descType;
	
	AEAddressDesc	finderAddress;
	
	AEDesc 			keyData;
	AEDesc 			nullDescriptor;
	AEDesc 			directObjectSpecifier;
	
	AEDesc 			dataDescriptor;
	AEDesc 			dataCEDescriptor;
	
	AEDesc			selectedItems;	

	// Get the address of the Finder and make a Get Data event.
	finderSerial = GetAddressOfFinder();
	AECreateDesc(typeProcessSerialNumber, &finderSerial, sizeof(finderSerial), &finderAddress);
	AECreateAppleEvent(kAECoreSuite, kAEGetData, &finderAddress, kAutoGenerateReturnID,
		kAnyTransactionID, &ae);	
	AEDisposeDesc(&finderAddress);
 
	// Make an object specifier for "selection" and put it into the
	// direct object of the event.
	descType = pSelection;
	AECreateDesc(typeType, &descType, sizeof(OSType), &keyData);
	AECreateDesc(typeNull, nil, 0, &nullDescriptor);
	CreateObjSpecifier(cProperty, &nullDescriptor, formPropertyID, &keyData, true, &directObjectSpecifier);
	AEPutParamDesc(&ae, keyDirectObject, &directObjectSpecifier);
	//clean-up
	AEDisposeDesc(&nullDescriptor);
	AEDisposeDesc(&directObjectSpecifier);
 
	// Put in the optional "requested type" parameter.
	descType = typeFSS;
	AECreateDesc(typeType, &descType, sizeof(OSType), &dataDescriptor);
	AECoerceDesc(&dataDescriptor, typeAEList, &dataCEDescriptor);
	AEPutParamDesc(&ae, keyAERequestedType, &dataCEDescriptor);
	//clean-up
	AEDisposeDesc(&dataCEDescriptor);
	AEDisposeDesc(&dataDescriptor);
 
	// Send the event, extract the reply, and dispose of event and reply.
	AESend(&ae, &reply, kAEWaitReply, kAENormalPriority, kAEDefaultTimeout, nil, nil);
	AEGetParamDesc(&reply, keyAEResult, typeAEList, &selectedItems);
	// need to do something with selectedItems
	//clean-up
	AEDisposeDesc(&reply);
	AEDisposeDesc(&ae);
	return(selectedItems);
}