#if powerc
#include <fp.h>
#endif	/*powerc*/

#include <Sound.h>
#include "SoundFileHeader.h"
#include "ReadHeader.h"
#include "OpenSoundFile.h"
#include "Main.h"
#include "Misc.h"
#include "Play.h"
#include "Menu.h"
#include "Dialog.h"
#include "RotateString.h"

#ifdef powerc
extern FileFilterUPP	gMyFileFilter;
#endif

extern GlowBalls	global;

extern MenuHandle	gFileMenu;
//extern DoubleBufPlayInfo	gDBPI;

StandardFileReply gOpenReply;

// if the vRefNum is set, skip the standard file shit, this file has already been found
short
OpenSoundFile(FSSpec fileSpec)
{		
	OSErr		error;
	Point		myPoint;
	SFTypeList	typeList;
		
	short		numTypes;
	Boolean		openAll = FALSE;
	FInfo		fndrInfo;
	Boolean	targetIsFolder,wasAliased;

	if(fileSpec.vRefNum == nil)
	{		
		SetDialogFont(systemFont);
		myPoint.h = -1;
		myPoint.v = -1;
		numTypes = -1;
#ifdef powerc
		StandardGetFile((FileFilterUPP)(gMyFileFilter), numTypes, typeList, &gOpenReply);
#else
		StandardGetFile((FileFilterUPP)(MyFileFilter), numTypes, typeList, &gOpenReply);
#endif
		if(gOpenReply.sfGood == FALSE)
		{
			return(FALSE);
		}
		StringCopy(gOpenReply.sfFile.name, global.theOpenSF.sfSpec.name);
		global.theOpenSF.sfSpec.vRefNum = gOpenReply.sfFile.vRefNum;
		global.theOpenSF.sfSpec.parID = gOpenReply.sfFile.parID;
	}
	else
	{
		BlockMoveData(fileSpec.name, global.theOpenSF.sfSpec.name, 64);
		global.theOpenSF.sfSpec.vRefNum = fileSpec.vRefNum;
		global.theOpenSF.sfSpec.parID = fileSpec.parID;
	}
	
	// Try to resolve the alias
	FSpGetFInfo(&global.theOpenSF.sfSpec, &fndrInfo);
	if(fndrInfo.fdFlags | 0x8000)
		error = ResolveAliasFile (&global.theOpenSF.sfSpec, TRUE, 
			&targetIsFolder, &wasAliased);
	if (targetIsFolder || error != 0)
		return(FALSE);

	error = FSpOpenDF(&(global.theOpenSF.sfSpec), fsCurPerm, &global.theOpenSF.dFileNum);
	if(error != 0 && error != opWrErr)
		return(FALSE);
	
	StringCopy("\p", global.theOpenSF.compName);
		
	// now we filter through different file types from most to least likely
	// but first separate out data fork less files, files without dataforks
	// sometimes fool the magic number tests randomly
		
	global.theOpenSF.sfType = FindSFType(global.theOpenSF.sfSpec);
	switch(global.theOpenSF.sfType)
	{
		case AIFF:
		case AIFC:
			error = ReadAIFFHeader(&global.theOpenSF);
			if(error != TRUE)
				error = ReadAIFCHeader(&global.theOpenSF);
			break;
		case AUDMED:
		case DSPD:
		case MMIX:
		case SDII:
			error = ReadSDIIHeader(&global.theOpenSF);
			break;
		case BICSF:
			error = ReadBICSFHeader(&global.theOpenSF);
			break;
		case NEXT:
		case SUNAU:
			error = ReadNEXTHeader(&global.theOpenSF);
			break;
		case WAVE:
			error = ReadWAVEHeader(&global.theOpenSF);
			break;
		case MPEG:
			error = ReadMPEGHeader(&global.theOpenSF);
			break;
		case RAW:	// if it matches nothing else, set this.
			global.theOpenSF.sRate = 44100.0;
			global.theOpenSF.nChans = 1;
			global.theOpenSF.dataStart = 0;
			global.theOpenSF.packMode = SF_FORMAT_16_LINEAR;
			global.theOpenSF.frameSize = 1;
			GetEOF(global.theOpenSF.dFileNum, &global.theOpenSF.numBytes);
			break;
	}
	if(global.theOpenSF.compName[0]==0)
		StringCopy("\pnot compressed", global.theOpenSF.compName);
		
	if(error != TRUE)
		return(FALSE);
	global.fileOpen = TRUE;
	global.fontChanged = TRUE;
	SetDialogFont(systemFont);
	return(TRUE);
}
void	HandlePlayClick(WindowPtr theWindow)
{
	EventRecord		localEvent;
	Point			newPosition;
	Boolean			fadeDone = FALSE;
	Rect			borderRect;
	long			controlCenterOffset;
	float			controlLength, frontLength;
	double			startTime;

	StopPlay(false);

	SetPort(theWindow);
	borderRect = theWindow->portRect;
	borderRect.left -= 5;
	borderRect.right += 5;
	controlCenterOffset = ((theWindow->portRect.right - theWindow->portRect.left) - 2) >> 1;
	while(fadeDone == FALSE)
	{
		WaitNextEvent(everyEvent, &localEvent, 0L, nil);
		switch(localEvent.what)
		{
			case mouseUp:
				GetMouse(&newPosition);
				fadeDone = TRUE;
				break;
			default:
				GetMouse(&newPosition);
				// when position moves out of the window, drop out
				if((newPosition.h >= borderRect.left) && (newPosition.h <= borderRect.right)
					&& (newPosition.v >= borderRect.top) && (newPosition.v <= borderRect.bottom))
				{
					if(newPosition.v <= (theWindow->portRect.top + controlCenterOffset))
						UpdatePlayWindow(&global.theOpenSF, 0);
					else if (newPosition.v >= (theWindow->portRect.bottom - controlCenterOffset))
						UpdatePlayWindow(&global.theOpenSF, (theWindow->portRect.bottom - (controlCenterOffset<<1)));
					else
						UpdatePlayWindow(&global.theOpenSF, (newPosition.v - controlCenterOffset));
				}
				else
					fadeDone = TRUE;
				break;
		}
	}
	controlLength = (float)((theWindow->portRect.bottom - theWindow->portRect.top) - (controlCenterOffset<<1));
	frontLength = global.theOpenSF.numBytes/(global.theOpenSF.sRate * global.theOpenSF.nChans * global.theOpenSF.frameSize);
	startTime = ((newPosition.v - controlCenterOffset) * frontLength)/controlLength;
	if(startTime <= 0.0)
		startTime = 0.0;
	if(startTime >= frontLength)
		startTime = frontLength;
	global.paused = FALSE;
	StartPlay(&global.theOpenSF, startTime, 0.0);
}

void	UpdatePlayWindow(SoundInfo *mySI, long position)
{
    GDHandle    	currGDevice;
	GrafPtr			oldPort;
    GWorldPtr		currGWorldPort;
	PixMapHandle	spritePM, playBarPM;
	RGBColor		fcolor, bcolor;
	Rect			drawRect, spriteRect, pictRect;
    OSErr			error;
	
	short pictNum;
	unsigned long secs;
		
	ShowWindow(global.theOpenSF.playWind);
    //Fetch the current port and gdevice and save them for later
	GetPort(&oldPort);
    GetGWorld(&currGWorldPort,&currGDevice);

	if(global.playBarGW == NIL_POINTER)
		return;
	playBarPM = GetGWorldPixMap(global.playBarGW);
	LockPixels(playBarPM);
	SetGWorld(global.playBarGW, NULL);	// Set port to the offscreen
	if(global.fontChanged)
	{
    //Dispose of the ptr
		DisposePtr(global.txtBitMap.baseAddr);
		TextFont(global.fontNum);
		TextSize(127);
		TextFace(0);
		TextMode(srcCopy);
		RotateString(mySI->sfSpec.name, &global.txtBitMap, clockWise);
		global.fontChanged = FALSE;
	}
	GetDateTime(&secs);
	fcolor.red = LoWord(secs<<3) + HiWord(secs);
	fcolor.blue = LoWord(secs<<2)+ HiWord(secs);
	fcolor.green = LoWord(secs<<3)+ HiWord(secs);
	RGBForeColor(&fcolor);
	bcolor.red = fcolor.red + 0x4000;
	bcolor.blue = fcolor.blue + 0x4000;
	bcolor.green = fcolor.green + 0x4000;
	RGBBackColor(&bcolor);
	
	drawRect = global.playBarGW->portRect;
	drawRect.right -= 1;
	
	EraseRect(&global.playBarGW->portRect);
	CopyBits(	&global.txtBitMap,
				(BitMap *) &(global.playBarGW->portPixMap),				//bitmap of gworld
				&global.txtBitMap.bounds,
				&drawRect,
				srcCopy+ditherCopy,
				nil
			);

	// Set up the rectangle for the play sprite gWorld
    spriteRect = global.playBarGW->portRect;
    spriteRect.left += 1;
    spriteRect.right -= 1;
    spriteRect.top += position;
    spriteRect.bottom = spriteRect.top + (spriteRect.right - spriteRect.left);
	
	pictRect.top = pictRect.left = 0;
	pictRect.right = pictRect.bottom = 64;
	
    //Set up our spirte gWorld
        //lock the offscreen buffer
	spritePM = GetGWorldPixMap(global.spriteGW);
	error = LockPixels(spritePM);
	if (!error)
		DebugStr("\pthe first LockPixels call failed");

    //Set the current graphics world to my offscreen source and draw into it
    SetGWorld((CGrafPtr) global.spriteGW,nil);
    EraseRect(&pictRect);
    pictNum = RangedRdm(0, 9);
    DrawPicture(global.playPict[pictNum],&pictRect);

	SetGWorld(global.playBarGW, NULL);	// Set port to the offscreen
    CopyBits(
                (BitMap *) &(global.spriteGW->portPixMap),
				(BitMap *) &(global.playBarGW->portPixMap),	//bitmap of gworld
                &pictRect,
                &spriteRect,
                subPin,
                nil
			);

    //Set the current graphics port to my application window for drawing into
    SetGWorld((CGrafPtr)global.theOpenSF.playWind,nil);
	CopyBits(
				(BitMap *) &(global.playBarGW->portPixMap),	//bitmap of gworld
				&(global.theOpenSF.playWind->portBits),
				&(global.playBarGW->portRect),				//portRect of gworld
				&(global.theOpenSF.playWind->portRect),		//portRect of window
				srcCopy, 
				nil 
			);
			
    //Unlock the offscreen buffers
    UnlockPixels(spritePM);
	UnlockPixels(playBarPM);

    //Restore the saved port and gdevice
    SetGWorld(currGWorldPort,currGDevice);

			
	SetPort(oldPort);
}

long
FindSFType(FSSpec fileSpec)
{
	FInfo	fndrInfo;
	long	sfType;
	Str255 testString;
	short i;

	FSpGetFInfo(&fileSpec,&fndrInfo);
	switch(fndrInfo.fdType)
	{
		case 'AIFF':
			sfType = AIFF;
			break;
		case 'AIFC':
			sfType = AIFC;
			break;
		case 'Sd2f':
			sfType = SDII;
			break;
		case 'DSPs':
			sfType = DSPD;
			break;
		case 'MSND':
			sfType = MMIX;
			break;
		case 'RIFF':
		case '.WAV':
		case 'WAVE':
			sfType = WAVE;
			break;
		case 'IRCM':
			sfType = BICSF;
			break;
		case 'NxTS':
		case 'ULAW':
			sfType = NEXT;
			break;
		case 'LMAN':
			sfType = LEMUR;
			break;
		case 'DATA':
			sfType = RAW;
			break;
		case 'MooV':
			sfType = QKTMV;
			break;
		case 'MPG3':
		case 'MPEG':
		case 'MPG ':
		case 'Mp3 ':
			sfType = MPEG;
			break;
		case 'SFIL':
			sfType = SDI;
			break;
		case 'SCRN':
		case 'CSCR':
		case 'PICT':
			sfType = PICT;
			break;
		case 'TEXT':
			sfType = TEXT;
			break;
		default:		//Unknown file type, match text
			sfType = RAW;	// if it matches nothing else, set this.

			for(i = 1; i <= 3; i++)
				testString[i] = fileSpec.name[i+(fileSpec.name[0] - 3)];
			testString[0] = 3;
			if(EqualString(testString,"\p.au",FALSE,TRUE))
				sfType = SUNAU;
			if(EqualString(testString,"\p.sf",FALSE,TRUE))
				sfType = BICSF;
			if(sfType == RAW)
			{
				for(i = 1; i <= 4; i++)
					testString[i] = fileSpec.name[i+(fileSpec.name[0] - 4)];
				testString[0] = 2;
				if(EqualString(testString,"\p.W",false,true))
					sfType = TX16W;
				if(EqualString(testString,"\p.w",false,true))
					sfType = TX16W;
				testString[0] = 4;
				if(EqualString(testString,"\p.snd",FALSE,TRUE))
					sfType = NEXT;
				if(EqualString(testString,"\p.irc",FALSE,TRUE))
					sfType = BICSF;
				if(EqualString(testString,"\p.wav",FALSE,TRUE))
					sfType = WAVE;
				if(EqualString(testString,"\p.pvc",FALSE,TRUE))
					sfType = CS_PVOC;
				if(EqualString(testString,"\p.aic",FALSE,TRUE))
					sfType = AIFC;
				if(EqualString(testString,"\p.aif",FALSE,TRUE))
					sfType = AIFF;
				if(EqualString(testString,"\p.mp3",FALSE,TRUE))
					sfType = MPEG;
			}
			if(sfType == RAW)
			{
				for(i = 1; i <= 5; i++)
					testString[i] = fileSpec.name[i+(fileSpec.name[0] - 5)];
				testString[0] = 5;
				if(EqualString(testString,"\p.mpeg",FALSE,TRUE))
					sfType = MPEG;
				if(EqualString(testString,"\p.aiff",FALSE,TRUE))
					sfType = AIFF;
				if(EqualString(testString,"\p.aifc",FALSE,TRUE))
					sfType = AIFC;
				if(EqualString(testString,"\p.sdif",FALSE,TRUE))
					sfType = SDIFF;
			}
			if(sfType == MPEG)
			{
				fndrInfo.fdType = 'MPG3';
				FSpSetFInfo(&fileSpec,&fndrInfo);
			}
			break;
	}
	return(sfType);
}

/* TRUE MEANS DON'T SHOW THE FILE */
pascal Boolean
MyFileFilter(CInfoPBPtr pBP)
{
	short i;
	Str255 testString;

	if(pBP->hFileInfo.ioFlAttrib & 0x10)	// Is it a directory?
		return(FALSE);
	if(pBP->hFileInfo.ioFlFndrInfo.fdType == 'Sd2f' || pBP->hFileInfo.ioFlFndrInfo.fdType == 'AIFF' || pBP->hFileInfo.ioFlFndrInfo.fdType == 'AIFC' ||
	   pBP->hFileInfo.ioFlFndrInfo.fdType == 'DSPs' || pBP->hFileInfo.ioFlFndrInfo.fdType == 'MSND' || pBP->hFileInfo.ioFlFndrInfo.fdType == 'RIFF' ||
	   pBP->hFileInfo.ioFlFndrInfo.fdType == '.WAV' || pBP->hFileInfo.ioFlFndrInfo.fdType == 'IRCM' || pBP->hFileInfo.ioFlFndrInfo.fdType == 'NxTS' ||
	   pBP->hFileInfo.ioFlFndrInfo.fdType == 'MP3 ' || pBP->hFileInfo.ioFlFndrInfo.fdType == 'Mp3 ' || pBP->hFileInfo.ioFlFndrInfo.fdType == 'ULAW')
		return(FALSE);
	for(i = 1; i <= 3; i++)
		testString[i] = pBP->hFileInfo.ioNamePtr[i+(pBP->hFileInfo.ioNamePtr[0] - 3)];
	testString[0] = 3;
	if(EqualString(testString,"\p.au",FALSE,TRUE))
		return(FALSE);
	if(EqualString(testString,"\p.sf",FALSE,TRUE))
		return(FALSE);
	for(i = 1; i <= 4; i++)
		testString[i] = pBP->hFileInfo.ioNamePtr[i+(pBP->hFileInfo.ioNamePtr[0] - 4)];
	testString[0] = 4;
	if(EqualString(testString,"\p.snd",FALSE,TRUE))
		return(FALSE);
	if(EqualString(testString,"\p.irc",FALSE,TRUE))
		return(FALSE);
	if(EqualString(testString,"\p.wav",FALSE,TRUE))
		return(FALSE);
	if(EqualString(testString,"\p.mp3",FALSE,TRUE))
		return(FALSE);
	if(EqualString(testString,"\p.mpg",FALSE,TRUE))
		return(FALSE);
	for(i = 1; i <= 5; i++)
		testString[i] = pBP->hFileInfo.ioNamePtr[i+(pBP->hFileInfo.ioNamePtr[0] - 5)];
	testString[0] = 5;
	if(EqualString(testString,"\p.aiff",FALSE,TRUE))
		return(FALSE);
	if(EqualString(testString,"\p.aifc",FALSE,TRUE))
		return(FALSE);
	if(EqualString(testString,"\p.mpeg",FALSE,TRUE))
		return(FALSE);
	return(TRUE);
}

